#-----------------------------------------------------------------------------
# MT only including Groups
# June 16, 2003
# revised Jan 7, 2005
# Copyright (c) 2005 by David G. Drumright (ockham)
#-----------------------------------------------------------------------------

import poser
import string
from Tkinter import *
root = Tk()
scene = poser.Scene()

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------

global OneActor

# First make sure there is a selected body part.
OneActor = scene.CurrentActor()
if not OneActor.IsBodyPart():
   raise "No selected body part"

#-----------------------------------------------------------------------------

def RunMorph(OrigName):
    GoodVerts1 = []
    GoodVerts2 = []

    # See if the parm is a MT
    OrigParm = OneActor.Parameter(OrigName)
    if not OrigParm.IsMorphTarget():
        raise "Selected parameter is not MT"

    # Get the Geom of the chosen actor:
    OneGeom = OneActor.Geometry()

    # Get the name of the first group to use in naming.
    GroupList = OneGeom.Groups()
    if not GroupList:
        raise "No groups in this actor"
    else:
        GroupName = GroupList[0] # Use this for naming

    # Go through the Polygons (facets) of the Geom.  For each,
    # see if it is included in any Group.  We don't care about
    # the details of Groups here; we only want to handle
    # the facets that are NOT in a Group.
    SetList = OneGeom.Sets()
    for OneFacet in OneGeom.Polygons():
        Grouped = 0
        for OneName in GroupList:
            if OneFacet.InGroup(GroupName): # This facet is not grouped
                Grouped=1
                break
        if Grouped:
            First = OneFacet.Start()
            Last = First + OneFacet.NumVertices()
            for VertIndex in SetList[First:Last]: # This is a list of vertex indexes in the poly
                # Add the vertices in this Group to the list of "good" verts.
                GoodVerts1.append(VertIndex)

    # Now filter the list to cut duplication.
    dummydict = {}
    for V in GoodVerts1:
        dummydict[V] = None # Let the dict internals do the sorting
    GoodVerts2 = dummydict.keys()
    # Form title, orig MT name plus Group
    NewName = OrigName + "INC" + GroupName

    OneActor.SpawnTarget(NewName)
    NewParm = OneActor.Parameter(NewName) # Fetch the new parm (Spawn doesn't return anything)

    # Now use the GoodVerts list to generate a new MT
    # using only the deltas corresponding to the GoodVerts.
    for OneVert in range(OneGeom.NumVertices()):
        if OneVert in GoodVerts2:
            X,Y,Z = OrigParm.MorphTargetDelta(int(OneVert)) # Get one delta from orig
        else:
            X,Y,Z = 0.0, 0.0, 0.0
        NewParm.SetMorphTargetDelta(OneVert, X,Y,Z)

#------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Include only Grouped sections in MT")

        # The leader info goes across the top.
        self.Label = Label(self.master,text="Pick MT from list:").grid(row=1, column=0)

        # Lists
        self.ListScroll = Scrollbar(self.master, orient=VERTICAL)
        self.ListScroll.grid( row=2, column=0,sticky=N+S+E)
        self.List = Listbox(self.master, height=5, width=40,selectmode=SINGLE,yscrollcommand = self.ListScroll.set)
        self.List.grid( row=2, column=1)
        self.ListScroll["command"] = self.List.yview
        # Fill the list with MTs on selected body part.
        ParmList = OneActor.Parameters()
        for OneParm in ParmList:
            if OneParm.IsMorphTarget():
                OneName = OneParm.Name()
                self.List.insert(END,OneName)


        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Go", command=self.handleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.handleCancel)
        self.buttonCancel.grid(row=3, column=2)

        # sample binding
        self.List.bind("<Double-Button-1>",self.handleSelect)

    # - - - - - - - - - - - - - - - - - -

    def handleSelect(self,event):
        # called when doubleclick in list.
        self.handleGo()  # just detour to regular

    # - - - - - - - - - - - - - - - - - -

    def handleGo(self):
        Selecteds = self.List.curselection() # Nominally tuple of strings, but this is SINGLE
        Name = self.List.get(int(Selecteds[0]))
        # Call the morph-maker for this parameter name
        RunMorph(Name)
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def handleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

app = App(root, "")
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
